#include<syscall.h>
#include<linux/types.h>
#include<linux/fs_acl.h>
#include<errno.h>
/*!
    Esta llamada utiliza la funcion cp_aclstat.
                                                                                                                                                                                      Stat devuelve informacion almacenada en la ACL de un archivo, siempre que el usuario que pide dicha informacion tiene permisos de lectura sobre el archivo.
    \param filename Archivo del cual se pide la informacion.
    \param statbuf Apunta a una estructura donde se devuelven los resultados de aclstat.
    \post
    - En el parametro statbuf quedan cargadas todas las entradas validas
    de la ACL del archivo filename. 

    \return
    - Un numero distinto de cero si filename no es un nombre valido \esp.
    - -EACCES Si el usuario que realiza la operacion no tiene permiso de lectura sobre el archivo filename \esp.
    - Lo que devuelva cp_aclstat en cualquier otro caso.
*/

int aclstat (char *filename, struct acl_entry *acl_statbuf, int len)
{
    int err = syscall (__NR_aclstat, filename, acl_statbuf, len);
    errno = -err;
    return err;
}

/*!
    Aclfstat devuelve informacion de la ACL del inodo  al que apunta el descripor de archivo que se pasa como parametro.

    \param fd Descriptor de archivo que apunta al inodo del cual se piede la informacion.
    \param statbuf Apunta a una estructura donde se devuelven los resultados de fstat.
    \post Idem aclstat
    \return Idem aclstat

*/

int aclfstat (unsigned int fd, struct acl_entry *acl_statbuf, int len)
{
    int err = syscall (__NR_aclfstat, fd, acl_statbuf, len);
    errno = -err;
    return err;
}


/*! 
    Quita permisos a un usuario o un grupo en la ACL del inodo de un archivo, siempre que esto sea posible.
    
    \param filename Nombre del archivo sobre el cual se realizara la operacion.
    \param id Identificacion del usuario/grupo al que se le quitan permisos en la ACL del inodo del archivo filename.
    \param mode Permisos que se le quitaran a los permisos que tenga el usuario/grupo id en la ACL del inodo del archivo filename. 
    \param type Determina si id es la identificacion de un usuario o de un grupo.
    \post La ACL del archivo filename cambia por una nueva ACL donde el 
    usuario/grupo con identificacio id tiene los permisos que tenia antes
    menos los permisos que estan en mode, el resto de la ACL queda sin cambios.
    Si id es el usuario dueno tradicional del archivo filename, y mode contiene el permiso de dueno, la ACL y el archivo cambian de la siguiente forma:
    El dueno tradicional de filename pasa a ser ROOT, se  agrega a la ACL el usuario ROOT con permisos de dueno. Se quitan de la ACL los permisos del usuario id que estan en mode.
    \return 
    - Un numero distinto de cero si filename no es un nombre valido \esp.
    - -EROFS Si el inodo del archivo filename es read-only.
    - -EPERM Si el inodo del archivo filename es inmutable o append?????
    - -EPERM Si se intenta borrar al Grupo ROOTGRP del conjunto de grupos duenos de la ACL del archivo filename \esp.
    - -EACCES Si el usuario que realiza la operacion no es dueno del archivo filename (\esp).    
    - -EBUSY Si el archivo filename esta abierto en el sistema \esp.
    - -EINVAL Si el usuario/grupo id no existia en la ACL del inodo del archivo filename antes de realizarse la operacion.
    - -ENOMEM Si la operacion no pudo agregar a ROOT en la ACL en el caso de que id sea el dueno tradicional de filename \esp. 
    - -ACL_OK Si la operacion se realizo con exito.
*/

int acldel (char *filename, int id, int mode, int type)
{
    int err = syscall (__NR_acldel, filename, id, mode, type);
    errno = -err;
    return err;
}

/*! 
    \param filename Nombre del archivo sobre el cual se realizara la operacion.
    \param id Identificacion del usuario/grupo al que se le agregan permisos a la ACL del inodo del archivo filename.
    \param mode Permisos que se le agregaran a los permisos que tenga el usuario/grupo id en la ACL del inodo del archivo filename.
    \param type Determina si id es la identificacion de un usuario o de un grupo.
    \post
    - Se agrega a la ACL del archivo filename el usuario/grupo con identificacion id y los permisos mode.
    \return 
    - Un numero distinto de cero si filename no es un nombre valido \esp.
    - -EROFS Si el archivo filename es read-only.
    - -EPERM Si el archivo filename es inmutable o append.???
    - -EACCES Si el usuario que realiza la operacion no es dueno del archivo filename \esp.
    - -EBUSY Si el archivo filename esta abierto en el sistema \esp.
    - -ACL_OK Si la operacion se realizo con exito.
    - -ENOMEM Si la ACL del inodo del archivo filename estaba llena y el usuario/grupo id no tenia ningun permiso en la ACL de este inodo antes de realizarse la operacion.
*/

int acladd (char *filename, int id, int mode, int type)
{
    int err = syscall (__NR_acladd, filename, id, mode, type);
    errno = -err;
    return err;
}
