#include<syscall.h>
#include<errno.h>
#include<unistd.h>
#include<sys/types.h>
#include<linux/sc.h>

/*!
    \param filename Archivo que contiene la SC que se utilizara para cargar los datos.
    \param secclass Clase de seguridad donde se va a cargar el resultado
    \result
	- Un numero distinto de hubo algun error.
	- 0 Si la operacion se realizo con exito
    \post
	- En el parametro secclass contiene la SC del archivo filename.

*/

int oscstat (char *filename, sc * secclass)
{

    int err;
    err = syscall (__NR_oscstat, filename, secclass);
    errno = -err;
    return err;

}


/*  Devuelve 1 si (level1, categories1) domina a (level2, categories2)
    Devuelve 0 si (level1, categories1) no domina a (level2, categories2)
*/

int sc_compare(sc *  sc1, sc *  sc2)
{
    if(sc1->level > sc2->level)
	return 0;
    if(~sc1->categories[0] & sc2->categories[0])
	return 0;    
    return 1;
}

/*!
    \param uid Identificacion del usuario que tiene la SC que se quiere obtener.
    \param secclass Donde se cargara  la SC del usuario uid.
    \post
    - El parametro secclass contiene la SC del usuario uid.
    \result
    -  Un numero distinto de cero si hubo algun error.
    -  o Si la operacion se realizo con exito.  
*/

int sscstat (uid_t uid, sc * secclass)
{
    int err = syscall (__NR_sscstat, uid, secclass);
    errno = -err;
    return err;
}




int sc_getsize(sc * sc){
    int i,q;
    q = 0;
    for (i = 0; i < NR_CATS; i++ )
	if ((sc->categories[0])  & (1<<i) )	
	    q++;  
    return q;
}


int sc_getlevel(sc * sc){
    return sc->level;
}


void sc_setlevel(sc * sc, unsigned int l){
    sc->level=l;
}


sc_cat_iterator sc_cat_first(sc * sc) {
	int i;
	for(i = 0; i < NR_CATS ; i++)
	    if ((sc->categories[0]) & (1<<i)) 	
		return i;	 

	return SC_IT_ERROR;
}


sc_cat_iterator sc_cat_next(sc * sc, sc_cat_iterator it){
    int i;
    for(i = it+1; (i < NR_CATS) ; i++)
	if ((sc->categories[0]) & (1<<i)) 	
	    return (sc_cat_iterator) i;	 

    return SC_IT_ERROR;
}



int sc_getcat (sc *sc, sc_cat_iterator it)
{
    return (int) it;
}

